//
//	ExtendedApp.m -- a class to make extra information available via the
//		application object.  You no longer need to link against ni_s.
//		Written by Don Yacktman (c) 1993 by Don Yacktman.
//				Version 1.1  All rights reserved.
//
//		This is a free object!  Contact the author for the latest version.
//		Don Yacktman, 4279 N. Ivy Lane, Provo, UT, 84604
//		e-mail:  Don_Yacktman@byu.edu
//
//		You may use and copy this class freely as long as you
//		comply with the following terms:
//			(1) Do not remove the author's name or any of the
//				copyright notices from this file.
//			(2) If you redistribute an application which uses this
//				object, you must either include the source code for
//				this object with the application or state in your
//				application's documentation that you (a) use this
//				object and (b) where to obtain the source code for
//				the object.
//			(3) In no way shall the author or his employer(s) be held
//				responsible for any damages caused by what this object
//				does or does not do.
//			(4) You have no warranty whatsoever that this object is
//				good for any purpose at all.  If you find it useful
//				for something, consider yourself lucky and leave it at that.
//

#import <daymisckit/daymisckit.h>
#import <mach/mach_error.h>
#import <strings.h>
#import <bsd/pwd.h>		// for getpwuid
#import <libc.h>		// for chdir, getwd, getuid

int DAYOSVersion() { // looks at kernel version string to get OS sub version
	// we could return 3.x as a float, but I am assuming version 3 or greater
	// since really the only info I need is if 3.1 additions are available
	// or not.  (GameKit needs to know which SoundKit stuff it can use...)
	kern_return_t       ret;
	kernel_version_t    string;
	id versString, temp;

	ret=host_kernel_version(host_self(), string);
	if (ret != KERN_SUCCESS) {
	    mach_error("host_kernel_version() call failed", ret);
		return(-1);
	}
	// use a DAYString to parse it; this is probably not the best way
	// to handle this since it expects the 3rd word to be "3.x:"
	// (but x can be any size, at least...)  the dependency is that
	// the version must be the 3rd word, and the last char should
	// be tossed.
	temp = [DAYString newWithString:string];
	versString = [temp wordNum:3]; [temp free];
	[versString removeFrom:([versString length] - 1) length:1];
	temp = [versString midFrom:2 length:([versString length] - 2)];
	ret = atoi([temp stringValue]); [temp free];
	return ret;
}

@implementation ExtendedApp

+ new;	// application uses new, not init.  Want to save path to app
{		// and obtain other misc. info after Application does it's init.
    self = [super new];
	gethostname(realHostName, MAXHOSTNAMELEN);
	userRealName = [self realNameFor:NXUserName()];
	return self;
}


- (const char *)appDirectory; { return [[NXBundle mainBundle] directory]; }
- (int)osVersion { return DAYOSVersion(); }
- (int)userIDNum { return getuid(); }
- (int)groupIDNum { return getgid(); }
- (int)effectiveUserIDNum { return geteuid(); }
- (int)effectiveGroupIDNum { return getegid(); }
- (unsigned long int)hostID { return gethostid(); }
- (NXAtom)userHomeDirectory { return NXHomeDirectory(); }
- (NXAtom)userLoginName { return NXUniqueString(NXUserName()); }
- (NXAtom)userRealName { return userRealName; }
- (NXAtom)realHostName { return NXUniqueString(realHostName); }

// Attempt to get the real name corresponding to user id.
// Thanks to shayman@objectario.com (Steve Hayman) for recommending
// using this over a NetInfo search.  This is much simpler...and
// I had forgotten that this function actually does the right thing
// for NetInfo, no NetInfo, YP, etc!  Much better solution...
- (NXAtom)realNameFor:(NXAtom)userId
{
	struct passwd *user = getpwuid(getuid()); // get the passwd file entry
	if (!user) return [self userLoginName]; // fall back on uid if error
	return NXUniqueString(user->pw_gecos);	// unique full name and return it
}

@end
